<?php

class Request_Cache
{
    protected $api = null;
    protected $cache = null;

    public function __construct()
    {
        // constructor for the class
        $this->api = MSPECS::getApi();
        $this->cache = MSPECS::getCache();
    }

    // get all deal tags for updating
    protected function tagDealData($dealId) {
        $tags = array('deals list', $dealId);
        // get cached deal and everything related to deal
        if ($deal = $this->api->getDealById($dealId)) {
            $tags[] = $deal->getCurrency();
            if ($files = $deal->getFiles()) {
                foreach ($files as $file) {
                    if ($file->getId()) {
                        $tags[] = $file->load()->getId();
                        $file->getUrl(true);
                    }
                }
            }
            if ($files = $deal->getPublicFiles()) {
                foreach ($files as $file) {
                    if ($file->getId()) {
                        $tags[] = $file->load()->getId();
                        $file->getUrl(true);
                    }
                }
            }
            if($links = $deal->getUrls()) {
                foreach($links as $link) {
                    if($link->getId()) {
                        $tags[] = $link->getId();
                    }
                }
            }
            if ($broker = $deal->getMainBroker()) {
                $tags[] = $broker->getId();
                if (($image = $broker->getProfileAvatar()) && ($imageId = $image->load()->getId())) {
                    $tags[] = $imageId;
                    $image->getImageUrl(true);
                    $image->getThumbnailUrl(true);
                }
            }

            if($bidding = $this->api->getBiddingById($deal->getBiddingId())) {
            $tags[] = $deal->getBiddingId();
            if($bids = $bidding->getBids()) {
                foreach ($bids as $bid) {
                        $tags[] = $bid->getId();
                    }
                }
            }

            if($viewings = $deal->getVisitingDates()) {
                foreach($viewings as $viewing) {
                    $tags[] = $viewing->getId();
                }
            }

            if($estate = $deal->getMainEstate()) {
                //recache easements and details
                if($easements = $estate->getEastemensAndDetails()) {
                    foreach($easements as $easement) {
                        $tags[] = $easement->getId();
                    }
                }

                //update tenants list
                if ($tenants = $estate->getTenants()) {
                    foreach ($tenants as $key => $value) {
                        $tags[] = $value->getId();
                    }
                }

                //recache farming
                if($estate->getObjectType() == 'OBJECT_TYPE_FARMING') {
                    if($areas = $estate->getAreas()) {
                        foreach($areas as $area) {
                            $tags[] = $area->getId();
                        }
                    }
                    //update farming specifics
                    if($farmingspecifics = $estate->getFarmingSpecifics()) {
                        foreach($farmingspecifics as $farmingSpecific) {
                            $tags[] = $farmingSpecific->getId();
                        }
                    }

                    if($mainPurposes = $estate->getMainPurposes()){
                        foreach($mainPurposes as $mainPurpose){
                            $tags[] = $mainPurpose->getId();
                        }
                    }
                }
                //recache operating costs
                if($operatingCosts = $estate->getOperatingCosts()) {
                    foreach($operatingCosts as $operatingcost) {
                        $tags[] = $operatingcost->getId();
                    }
                }
                //recache operating costs
                if($commercialOtherIncomes = $estate->getCommercialOtherIncomes()) {
                    foreach($commercialOtherIncomes as $commercialOtherIncome) {
                        $tags[] = $commercialOtherIncome->getId();
                    }
                }

                //recache everything for housing asos
                if($housingAsos = $estate->getHousingAssociation()) {
                    $tags[] = $housingAsos->getId();
                }

                if($patios = $estate->getPatios()) {
                    foreach($patios as $patio) {
                        $tags[] = $patio->getId();
                    }
                }

                if($parkings = $estate->getParkingSpaces()) {
                    foreach($parkings as $parking) {
                        $tags[] = $parking->getId();
                    }
                }
                //recache buildings
                if($estate->getObjectType() == 'OBJECT_TYPE_FARMING' || $estate->getObjectType() == 'OBJECT_TYPE_COMMERCIAL') {
                    if($buildings = $estate->getBuildings()) {
                        foreach($buildings as $building) {
                            $tags[]= $building->getId();
                        }
                    }
                }
                //recache mortgages
                if($mortgages = $estate->getMortgages()) {
                    foreach($mortgages as $mortgage) {
                        $tags[] = $mortgage->getId();
                    }
                }
                //recache rooms
                if($rooms = $estate->getRooms()) {
                    foreach($rooms as $room) {
                        $tags[] = $room->getId();
                    }
                }

                //recache premises
                if($premises = $estate->getPremises()) {
                    foreach($premises as $premise) {
                        $tags[] = $premise->getId();
                    }
                }

                if($images = $estate->getPublicImages()) {
                    foreach($images as $image) {
                        if($image->getId()) {
                            $tags[] = $image->load()->getId();
                            $image->getImageUrl(true);
                            $image->getThumbnailUrl(true);
                        }

                    }
                }

                $categories = array(null, 'ENUMS_IMAGES_IN_ESTATES_CATEGORY_FLOOR_PLAN');
                // get floor plan cache
                foreach ($categories as $category) {
                    if ($images = $estate->getImages($category)) {
                        foreach ($images as $image) {
                            if ($imageId = $image->load()->getId()) {
                                $tags[] = $imageId;
                                $image->getImageUrl(true);
                                $image->getThumbnailUrl(true);
                            }
                        }
                    }
                }

                if($deal->getObjectType() == 'OBJECT_TYPE_HOUSING_DEVELOPMENT'){
                    if($newDevelopmentDeals = $deal->getNewDevelopmentDeals()){
                        foreach ($newDevelopmentDeals as $newDevelopmentDeal) {
                            $tags[] = $newDevelopmentDeal->getId();
                        }
                    }
                }

                if($deal->getObjectType() == 'OBJECT_TYPE_PREMISE'){
                    if($extras = $estate->getPremiseExtras()){
                        foreach ($extras as $extra){
                            $tags[] = $extra->getId();
                        }
                    }
                }
            }
        }

        return $tags;
    }

    // recache deal and everything related to this deal
    protected function reCacheDeal($dealId) {

        // $tags = $dealId;
        // turn off api request
        $this->api->setForceCached(true);
        $this->api->setCallRequest(true);

        // get all deal tags
        $tags = $this->tagDealData($dealId);
        // turn on request api
        $this->api->setForceCached(false);
        $this->api->setCallRequest(false);
        //echo '<pre>' . print_r($tags, true) . '</pre>';
        $this->cache->updateCacheByTags($tags, array($this, 'recache'), 'any');
    }

    // before doing update, we need to check if the deal is accessable on server or not
    protected function _isDealAccessable($dealId) {
        $this->api->setNoCacheCall(true);
        $available = false;
        if($data = $this->api->getDealById($dealId)) {
            $available = true;
        } else {
            $available = false;
        }
        $this->api->setNoCacheCall(false);
        return $available;
    }

    public function updateDealsList($dealId) {
        $args = array(
            'api' => 'getDeals',
            'query' => array(
                'q' => "isPublished=true and objectStatus IN ('ENUMS_OBJECTSTATUS_TYPE_IN_PROGRESS', 'ENUMS_OBJECTSTATUS_TYPE_FOR_SALE') and isInternational=false",
                'sort' => '-updatedDate',
                'view' => 'deals'
            )
        );

        $deals = $this->api->getDeals($args);
        // MSPECS::GetHelper()->printJson($deals);
    }

    // remove all deal cache date
    public function deleteLocalDeal($dealId) {
        // turn off api request
        $this->api->setForceCached(true);
        $this->api->setCallRequest(true);
        $this->api->setForceDelete(true);
        // everything that has been tagged is deleted
        $tags = $this->tagDealData($dealId);
        $this->updateDealsList($dealId);
        // also get list of the deals
        // turn on request api
        $this->api->setForceCached(false);
        $this->api->setCallRequest(false);
        $this->api->setForceDelete(false);
        $content = array('code'=>'200', 'status'=>'success', 'message'=>"deal with id {$dealId} is deleted");
        ignore_user_abort(true);
        set_time_limit(0);
        ob_start();
        echo json_encode($content, JSON_PRETTY_PRINT);
        header('Connection: close');
        header('Content-Type: application/json');
        header('Content-Length: '.ob_get_length());
        ob_end_flush();
        ob_flush();
        flush(); // releasing the browser from waiting
    }

    // running all queries with api class. this should recache everything
    // basically it does simple requests. it's just we cleaned up old cache
    // this method has to be public to be able to bind for cache class methods
    public function recache($file, $content)
    {
        @unlink($file);
        if(@$content['info']['url']){
            $this->api->query($content['info']['url']);
        }
    }

    // handles deal update and also makes async response
    public function updateDealRequest($dealId) {
        if($this->_isDealAccessable($dealId)) {
            //detach client here
            ignore_user_abort(true);
            set_time_limit(0);
            $content = array('code'=>'200', 'status'=>'success', 'message'=>"deal with id {$dealId} is updating now");
            ob_start();
            echo json_encode($content, JSON_PRETTY_PRINT);
            header('Connection: close');
            header('Content-Type: application/json');
            header('Content-Length: '.ob_get_length());
            ob_end_flush();
            ob_flush();
            flush(); // releasing the browser from waiting
            $this->reCacheDeal($dealId);
        } else {
            //detach client here
            ignore_user_abort(true);
            $content = array('code'=>'401', 'status'=>'error', 'message'=>"no access to {$dealId}");
            ob_start();
            echo json_encode($content, JSON_PRETTY_PRINT);
            header('Connection: close');
            header('Content-Type: application/json');
            header('Content-Length: '.ob_get_length());
            ob_end_flush();
            ob_flush();
            flush();
        }
    }

    // updates everything related to bidding
    public function updateBidding($biddingId) {
        $api = $this->api;
        $cache = $this->cache;
        $tags = array();

        $api->setForceCached(true);
        $api->setCallRequest(true);

        $tags[] = $biddingId;

        if($bidding = $api->getBiddingById($biddingId)) {
            if($bids = $bidding->getBids()) {
                foreach ($bids as $bid) {
                    $tags[] = $bid->getId();
                }
            }
        }

        $api->setForceCached(false);
        $api->setCallRequest(false);

        // echo '<pre>' . print_r($tags, true) . '</pre>';
        $cache->updateCacheByTags($tags, array($this, 'recache'), 'any');
        return $tags;
    }

    // updates organization and list of brokers in this organization
    public function updateCompanyTree() {
        $api = $this->_getApi();
        $cache = $this->cache;
        $tags = array();
        $api->setForceCached(true);
        $api->setCallRequest(true);

        $tags[] = 'organizations';
        if($company = $api->getCompany()) {
            $tags[] = $company->getId();
        }

        //add brokers tags
        //get broker access right id
        $accessRight = $api->getBrokers();
        $tags[] = 'contacts/getByAccessRight/' . $accessRight['id'];

        if($brokers = $api->getBrokers()) {
            foreach($brokers as $broker) {
                $tags[] = $broker->getId();
            }
        }

        $api->setForceCached(false);
        $api->setCallRequest(false);

        //echo '<pre>' . print_r($tags, true) . '</pre>';
        $cache->updateCacheByTags($tags, array($this, 'recache'), 'any');
        //update deals list
        //$cache->updateCacheByTags("deals", array($this, 'recache'));
        return $tags;
    }

}