<?php

/*
  Plugin Name: MSPECS API
  Description: Deprecated plugin to display data from MSPECS
  Version: 2.6.0
  Author: MSPECS AB
  Author Email: info@mspecs.se
 */

if (!defined('ABSPATH')) {
    die();
}

//setting date and language code, used e.g. in model.php in function getFormattedDate()
date_default_timezone_set('Europe/Stockholm');
setlocale(LC_ALL, "sv_SE.UTF-8");
define("MSPECS_PLUGIN_URL", plugin_dir_url(__FILE__));

/*
    Gets the version number defined in the comment above.
    The comment above is where wordpress gets all the info about the
    plugin. Since the version has to be defined there we must
    get the version number from there.
*/
$pluginData = get_file_data(__FILE__, array("Version" => "Version"));
define("MSPECS_PLUGIN_VERSION", $pluginData['Version']);

if (!defined('MSPECS_THEME')) {
        define('MSPECS_THEME', 'mspecs-standard');
}

class MSPECS
{

    protected static $_api = null;
    protected static $_post = null;
    protected static $_cache = null;
    protected static $_helper = null;
    protected static $_config = null;
    protected static $_sysconfig = null;
    protected static $_requestCache = null;
    protected static $_table = 'clientUsers';
    protected static $_db_version = '1.0';

    public static function getApi(){
        if (!self::$_api) {
            //get api parameters form DB
            $apiLink = self::getParam('link');
            $username = self::getParam('username');
            $password = self::getParam('password');

            self::$_api = new Mspecs_Api($apiLink, $username, $password);
        }
        return self::$_api;
    }

    public static function getPost() {
        if(!self::$_post) {
            self::$_post = new Mspecs_Post();
        }
        return self::$_post;
    }

    public static function getConfig() {
        if(!self::$_config) {
            self::$_config = new Conf();
        }
        return self::$_config;
    }

    public static function getRequestCaching() {
        if (!self::$_requestCache) {
            self::$_requestCache = new Request_Cache();
        }
        return self::$_requestCache;
    }

    public static function getCache()
    {
        if (!self::$_cache) {
            self::$_cache = new Mspecs_Cache();
        }
        return self::$_cache;
    }

    public static function getHelper()
    {
        if (!self::$_helper) {
            self::$_helper = new Mspecs_Helper();
        }
        return self::$_helper;
    }

    //Updates the database with the new configuration or inserts a new row if the table is empty. It should only exist one row in this table.
    public static function setConfig($config){
        global $wpdb;

        //Convert from HTML values to real ones on the checkboxes
        $config["SMTPAuth"] = isset($config["SMTPAuth"]) && $config["SMTPAuth"] == "on" ? true : false;
        $config["useGoogleCaptcha"] = isset($config["useGoogleCaptcha"]) && $config["useGoogleCaptcha"] == "on" ? true : false;

        //If these fields have no value we keep the value that is already in the database
        if(isset($config["password"]) && $config["password"] == "")
            unset($config["password"]);
        if(isset($config["encryptionKey"]) && $config["encryptionKey"] == "")
            unset($config["encryptionKey"]);
        if(isset($config["internalToken"]) && $config["internalToken"] == "")
            unset($config["internalToken"]);

        $tableName = $wpdb->prefix . "config";
        $row = $wpdb->get_row("Select * from $tableName");

        //The table will only have one row
        if($wpdb->num_rows > 0)
            $wpdb->update($tableName, $config, array("id" => $row->id));
        else
            $wpdb->insert($tableName, $config);

        self::$_config = new Conf();

    }

    public static function getOption($option)
    {
        return get_option('mspecsapi:' . $option, '');
    }

    public static function getViewPath($view)
    {
        if(file_exists($viewPath = self::getViewFolder(constant("MSPECS_THEME")) . $view . '.phtml')) {
            return self::getViewFolder(constant("MSPECS_THEME")) . $view . '.phtml';
        }

        return self::getViewFolder() . $view . '.phtml';
    }

    public static function getViewFolder($theme = "mspecs-standard"){
        if($theme == "mspecs-premium") {
            return plugin_dir_path(__FILE__) . 'mvc/views-premium/';
        }

        return plugin_dir_path(__FILE__) . 'mvc/views/';
    }

    public static function setOption($option, $value)
    {
        update_option('mspecsapi:' . $option, $value);
    }

    public static function getParam($option) {
        global $wpdb;
        $table_name = $wpdb->prefix . self::$_table;
        $site_url = get_bloginfo('url');
        $relative_site_url = preg_replace('#^https?://#', '/', $site_url);
        $row = $wpdb->get_row($wpdb->prepare("SELECT {$option} FROM {$table_name} WHERE site LIKE %s", "%$relative_site_url%"));
        if(!empty($row)) {
            return $row->{$option};
        }
        else {
            return null;
        }
    }

    public static function setParam($key, $value) {
        global $wpdb;
        $site_url = get_bloginfo('url');
        $table_name = $wpdb->prefix . self::$_table;
        $wpdb->update($table_name, array($key => $value), array('site' => $site_url));
    }

    public static function apiCredidentialsExists(string $apiLink=null, string $username=null, string $password=null){
        return !empty($apiLink) && !empty($username) && !empty($password);
    }

    public static function register()
    {
        register_activation_hook(__FILE__, __CLASS__ . '::activatePlugin');

        self::includeFile('vendor/autoload.php');
        self::initDBcheck();

        add_action('admin_notices', __CLASS__ . '::NotifyNoCredidentials');

        //The backend must be initiated on the wp_loaded action so that it can register all wp-ajax routes
        add_action("wp_loaded", __CLASS__ . '::initBackend');

        // Remove canonical url that displays wrong with this setup
        remove_action('wp_head', 'rel_canonical');

		add_action( 'wp_head', __CLASS__ . '::initFrontendHeader' );

        //The frontend must be initiated on the wp action which is after wp_loaded so
        //that it can now if a request is an rest request or not
        add_action('wp', __CLASS__ . '::initFrontend');
    }

    public static function initFrontendHeader() {
        if(defined('MSPECS_NO_INDEX') && constant("MSPECS_NO_INDEX") == 1) {
            echo '<meta name="robots" content="noindex">';
        }
        // Add og image for preview
        if(defined('MSPECS_OG_IMAGE')) {
            echo '<meta property="og:image" content="' . constant('MSPECS_OG_IMAGE') . '" />';
        }
    }

    public static function initBackend(){
        if(is_admin()){
            self::includeFile('mvc/controllers/admin/settings.php');
            self::includeFile('mvc/controllers/ajax.php'); // ajax is frontend, but wp loads it as admin
            self::includeFile('mvc/controllers/booking.php'); // booking is frontend, but wp loads it as admin
            self::includeFile('mvc/controllers/viewing.php'); // viewing is frontend, but wp loads it as admin
            self::includeFile('mvc/controllers/mailer.php'); // mailer is frontend, but wp loads it as admin
            self::includeFile('mvc/controllers/updateTranslations.php');
        }
    }

    public static function initFrontend(){
        if((defined("REST_REQUEST") && REST_REQUEST) || is_admin()){
            return;
        }

        $apiLink = self::getParam('link');
        $username = self::getParam('username');
        $password = self::getParam('password');

        if(!self::apiCredidentialsExists($apiLink, $username, $password)){
            if(get_locale() == "sv_SE")
                $messageText = "Du måste fylla i dina inloggningsuppgifter till MSPECS API. Det kan du göra på <b>MSPECS API</b> under <b>Inställningar</b> i adminsidorna.";
            else
                $messageText = "You must enter your credidentials for the MSPECS API. You can do that on <b>MSPECS API</b> under <b>Settings</b> in the admin pages.";

            wp_die($messageText);
        }

        $translation = new Translation();
        $translation->createTranslations();
        $GLOBALS["translations"] = $translation;

        self::includeFile('mvc/controllers/frontend.php');
        self::includeFile('mvc/controllers/shortcode.php');
        add_action('wp_enqueue_scripts', __CLASS__ . '::registerScripts');
    }

    public static function activatePlugin() {
        MSPECS::createDB();
        MSPECS::insertRowDB();
        MSPECS::insertPreviousData();
    }

    //Credit: https://wptheming.com/2011/08/admin-notices-in-wordpress/
    //Adds a message box on the plugins site if no credidentials have been added
    public static function NotifyNoCredidentials(){
        global $pagenow;
        if ( $pagenow == 'plugins.php' ) {
            $apiLink = self::getParam('link');
            $username = self::getParam('username');
            $password = self::getParam('password');
            if(!self::apiCredidentialsExists($apiLink, $username,$password)){
                $messageText;
                //Temporary translation. Should be changed with the built in translation methods.
                if(get_locale() == "sv_SE")
                    $messageText = "Du måste fylla i dina inloggningsuppgifter till MSPECS API. Det kan du göra på <b>MSPECS API</b> under <b>Inställningar</b>.";
                else
                    $messageText = "You must enter your credidentials for the MSPECS API. You can do that on <b>MSPECS API</b> under <b>Settings</b>.";

                echo '<div class="updated"><p>' . $messageText . '</p></div>';
            }
        }
    }

    //Creates a new table in the database if none exist. Otherwise it updates the structure of the existing database. All this is done by the dbDelta method
    public static function createDB() {

        global $wpdb;
        $table_name = $wpdb->prefix . self::$_table;

        $charset_collate = '';
        if ( ! empty( $wpdb->charset ) ) {
            $charset_collate = "DEFAULT CHARACTER SET {$wpdb->charset}";
        }

        if ( ! empty( $wpdb->collate ) ) {
            $charset_collate .= " COLLATE {$wpdb->collate}";
        }

        $sql = "CREATE TABLE $table_name (
        id mediumint(9) NOT NULL AUTO_INCREMENT,
        username varchar(100) DEFAULT '' NOT NULL,
        password varchar(50) DEFAULT '' NOT NULL,
        link text NOT NULL,
        site text NOT NULL,
        name text,
        theme text,
        noIndex tinyint(1) NOT NULL DEFAULT '0',
        PRIMARY KEY (id)
        ) $charset_collate;";

        $table_name = $wpdb->prefix . "config";
        $config = "CREATE TABLE $table_name (
        id INT NOT NULL AUTO_INCREMENT ,
        encryptionKey VARCHAR(255) NOT NULL ,
        internalToken VARCHAR(255) NOT NULL ,
        server VARCHAR(60) NOT NULL ,
        username VARCHAR(255) NOT NULL,
        password VARCHAR(255) NOT NULL,
        port VARCHAR(10) NOT NULL ,
        charSet VARCHAR(60) NOT NULL ,
        SMTPSecure VARCHAR(15) NOT NULL ,
        SMTPAuth BOOLEAN NOT NULL ,
        useGoogleCaptcha BOOLEAN NOT NULL ,
        secretKey VARCHAR(255) NOT NULL ,
        siteKey VARCHAR(255) NOT NULL ,
        googleStaticMapKey VARCHAR(255) NOT NULL ,
        PRIMARY KEY (id)) $charset_collate;";

        require_once( ABSPATH . 'wp-admin/includes/upgrade.php' );
        dbDelta( $sql );
        dbDelta($config);
        //keep table version in options
        add_option( 'mspecs_db_version', self::$_db_version );
    }

    public static function initDBcheck() {
        //check if there is a row we need
        $row = MSPECS::getParam('site');
        $row2 = MSPECS::getParam('theme');
        if(empty($row)) {
            //there is no table, so let's migrate everything from previous version of plugin
            MSPECS::createDB();
            MSPECS::insertRowDB();
            MSPECS::insertPreviousData();
        } else if(empty($row2)) {
            MSPECS::createDB();
        }

        global $wpdb;
        if($wpdb->get_row("SHOW TABLES LIKE '" . $wpdb->prefix . "config';") == null)
            MSPECS::createDB();
    }

    public static function insertRowDB() {
        global $wpdb;
        $table_name = $wpdb->prefix . self::$_table;
        $siteurl = get_bloginfo('url');

        $wpdb->insert($table_name, array('username'=>'', 'password'=>'', 'link'=>'', 'site'=>$siteurl), array('%s', '%s', '%s', '%s'));
    }

    public static function insertPreviousData() {
        $apiLink = self::getOption('api-link');
        $username = self::getOption('username');
        $password = self::getOption('password');

        if(self::apiCredidentialsExists($apiLink, $username,$password)) {
            MSPECS::setParam('username', $username);
            MSPECS::setParam('password', $password);
            MSPECS::setParam('link', $apiLink);
        }
    }

    public static function registerScripts()
    {
		wp_enqueue_style('print:css', MSPECS_PLUGIN_URL . 'includes/print.css');
        //wp_enqueue_style('user:css', MSPECS_PLUGIN_URL . 'includes/user_css.php?' . http_build_query(self::getUserColors())); // MOVED TO THEME
        wp_enqueue_style('roboto:css', '//fonts.googleapis.com/css?family=Roboto:400,300,500,500i,400i,300i,700');
        wp_enqueue_script('mspecs:js', 'https://www.google.com/recaptcha/api.js?onload=onRecaptchaLoad&render=explicit');
        wp_enqueue_script('intTelInputUtils:js', MSPECS_PLUGIN_URL . 'includes/intTelInput/js/utils.js',  array(), false, true);
        wp_enqueue_style('intTelInput:css', MSPECS_PLUGIN_URL . 'includes/intTelInput/css/intlTelInput.min.css');
        wp_enqueue_script('intTelInput:js', MSPECS_PLUGIN_URL . 'includes/intTelInput/js/intlTelInput.min.js', array(), false, true);
        wp_enqueue_script('script', MSPECS_PLUGIN_URL . 'includes/mspecs.js', array('jquery'), 1.3, true);
        // wp_enqueue_script( 'script', '/wp-content/plugins/mspecs/includes/mspecs.js', array ( 'jquery' ), 1.1, true);
    }

    public static function includeFile($path)
    {
        foreach (glob(plugin_dir_path(__FILE__) . $path) as $file) {
            if (is_file($file)) {
                include_once($file);
            }
        }
    }

    public static function getUserColors()
    {
        $api = self::getApi();
        return $api->getUserColors();
    }

}

MSPECS::includeFile('classes/*.php');
MSPECS::includeFile('mvc/models/*.php');

MSPECS::register();
