<?php

/***
 * PLEASE DO NOT CHANGE ANY CODE BELOW
 */

class Mspecs_Controller_Ajax extends Mspecs_Controller
{

    protected $api = null;
    protected $content = null;
    protected $debugMsg = null;
    protected $_requestCache = null;

    public function __construct()
    {
        add_action('wp_ajax_mspecsapi', array($this, 'render'));
        add_action('wp_ajax_nopriv_mspecsapi', array($this, 'render'));
        $this->_requestCache = MSPECS::getRequestCaching();
    }

    protected function cache($attr)
    {
        set_time_limit(0);

        $cache = MSPECS::getCache();

        if (isset($attr['clear'])) {
            $content = null;
            switch ($attr['clear']) {
                case 'all':
                    $cache->clearCache();
                    $cache->clearFiles();
                    break;
                case 'data':
                    $content = $cache->clearCache();
                    break;
                case 'deal':
                    if (empty($attr['deal']))
                        throw new Exception('Deal is not set');
                    // $content = $this->_updateDeal($attr['deal']);
                    $content = $this->_requestCache->updateDealRequest($attr['deal']);
                    break;
                case 'bidding':
                    if (empty($attr['bidding']))
                        throw new  Exception("Bidding is not set");
                    $content = $this->_requestCache->updateBidding($attr['bidding']);
                    break;
                case 'file':
                    if (empty($attr['file']))
                        throw new Exception('File is not set');
                    if (!preg_match('#^https?://[^/]+/.+#i', $attr['file']))
                        throw new Exception('File URL is incorrect');
                    $content = $cache->getFileUrl($attr['file'], true);
                    break;
                case 'files':
                    $content = $cache->clearFiles();
                    break;
                case 'object':
                    if (empty($attr['object']))
                        throw new Exception('Object is not set');
                    //also update company tree
                    $this->_requestCache->updateCompanyTree();
                    $content = $cache->updateCacheByTags($attr['object'], array($this, 'recache'));
                default:
            }
            return $content;
        }
        return '';
    }

    protected function deleteDeal($attr) {
        if (isset($attr['clear'])) {
            switch ($attr['clear']) {
                case 'deal':
                if (empty($attr['deal']))
                    throw new Exception('Deal is not set');
                return $this->_requestCache->deleteLocalDeal($attr['deal']);
                default:
                throw new Exception("'api' attribute value '{$attr['clear']}' is not supported");
            }
        }
    }

    protected function deleteRequest() {
        $attr = $_REQUEST;
        switch ($attr['api']) {
            case 'cache':
            $useJson = true;
            $this->deleteDeal($attr);
            break;
            default:
            throw new Exception("'api' attribute value '{$attr['api']}' is not supported");
        }
    }

    protected function getRequest()
    {
        $attr = $_REQUEST;
        $content = '';
        $vars = array();
        $useJson = true;
        $useTemplate = true;

        try {
            if (empty($attr['api']))
                throw new Exception("'api' attribute is not set");

            //With these apiCalls you can retreive the html generated by the specified functions. For example, if you call getDeals you will get the html for displaying all the deals.
            $apiCalls = array(
                'getBrokers' => 'brokers',
                'getCompany' => 'company',
                'getDeals' => 'deals',
                'getEstates' => 'estates',
                'getUrls' => 'urls',
                'getWebhome' => 'webhome',
            );
            switch ($attr['api']) {
                case 'cache':
                    $useJson = true;
                    $useTemplate = false;
                    $content = $this->cache($attr);
                    break;
                case 'dealPage':
                    if(empty($attr['deal']))
                        throw new Exception("Deal ID is not set");
                    if(!$model = $this->api->getDealById($attr['deal']))
                        throw new Exception("Deal ID is not found");
                    $content = $model->getViewUrl();
                    wp_redirect($content); exit;
                    break;
                case 'dealImage':
                    if(empty($attr['deal']))
                        throw new Exception("Deal ID is not set");
                    if (!$model = $this->api->getDealById($attr['deal']))
                        throw new Exception("Deal ID is not found");
                    $content = $model->getImageViewUrl();
                    wp_redirect($content); exit;
                    break;


                default:
                    if (isset($apiCalls[$attr['api']])) {
                        $vars[$apiCalls[$attr['api']]] = $this->api->{$attr['api']}($attr);
                    } else {
                        throw new Exception("'api' attribute value '{$attr['api']}' is not supported");
                    }
            }

            if ($useTemplate) {
                if (empty($attr['view']))
                    throw new Exception("Template file is not set");
                $content = $this->_renderView($attr['view'], $vars);
            }

            $output = array(
                'status' => 'success',
                'content' => $content,
            );
        } catch (Exception $e) {
            $output = array(
                'status' => 'error',
                'message' => $e->getMessage(),
            );
        }

        //this will work only if the deal update is called
        //this is async. after response it will continue updating the deal
        if($attr['clear'] && $attr['clear']=='deal') {
            exit;
        } else {
            if ($useJson) {
                header('Content-Type: application/json');
                echo json_encode($output);
            } else {
                if ($output['status'] == 'error') {
                    header('HTTP/1.1 500 Internal Server Error', true, 500);
                    echo $output['message'];
                } else {
                    header('Content-Type: application/json');
                    echo $output['content'];
                }
            }
            exit;
        }
    }

    public function render() {
        $methodType = $_SERVER['REQUEST_METHOD'];

        $this->api = $this->_getApi();
        $this->api->setForceCached(false);

        switch ($methodType) {
            case 'DELETE':
            $this->deleteRequest();
            break;
            case 'GET':
            default:
            $this->getRequest();
            break;
        }
    }

    public function retrieveCredentials($attr) {
        if($attr['token'] == $this->_getApi()->getInternalToken()) {
            $credentials['username'] = $this->_getApi()->getUser();
            $credentials['password'] = $this->_getApi()->getPassword();
            return $credentials;
        }
        else {
            $credentials['msg'] = 'wrong token';
        }
        return $credentials;
    }
}

$ajaxController = new Mspecs_Controller_Ajax();
