<?php

class Mspecs_Model_Deal extends Mspecs_Model
{

    protected $_files = null;
    protected $_publicFiles = null;
    protected $_urls = null;
    protected $_mainEstate = null;
    protected $_currency = null;
    protected $_currencyForeign = null;
    protected $_viewings = array();

    public function __construct($data = array())
    {
        if (is_array($data)) {
            $this->setData($data);
        }
        $this->setCurrencies();
    }

    public function setCurrencies()
    {
        if($currencyId = $this->getCurrency()) {
            $query = array(
                'path' => 'currencies',
                'query' => array(
                    'q' => "id='{$currencyId}'"
                ),
            );
            $data = $this->_getApi()->queryOne($query);
            $this->_currency = $data['isoCode'];
        }

        if($foreignCurrencyId = $this->getForeignStartingPriceCurrencyId()) {
            $query2 = array(
                'path' => 'currencies',
                'query' => array(
                    'q' => "id='{$foreignCurrencyId}'"
                ),
            );
            $data2 = $this->_getApi()->queryOne($query2);
            $this->_currencyForeign = $data2['isoCode'];
        }

    }

    public function getMainImage()
    {
        if ($imageId = $this->getMainImageFileId()) {
            return new Mspecs_Model_Image($imageId);
        }
        return null;
    }

    public function setOgImage($url) {
        define('MSPECS_OG_IMAGE',get_site_url() . $url);
    }

    public function getDealImages()
    {
      $query = array(
        'path' => 'imagesInDeals',
        'query' => array(
          'q' => "dealId='{$this->getId()}'"
        )
      );
      if ($data = $this->_getApi()->query($query)) {
        $imagesInDeals = array();
        if (!empty($data)) {
          $imagesInDeals = array_map(function($obj) {
            return $obj['fileId'];
          }, $data);
        }

        if ($imagesData = ($this->_getApi()->query(array(
          'path' => 'files',
          'query' => array(
            'q' => "id IN ('" . join("','", $imagesInDeals) . "') AND isPublished=true",
            'sort' => 'displayOrder'
          )
        )))) {
          $images = array();
          foreach ($imagesData as $key => $image) {
            array_push($images, new Mspecs_Model_Image($image));
          }
          return new Mspecs_Collection($images);
        }
      }
      return null;
    }

    public function getLeadForwardEmail()
    {
        if($this->getOrganizationUnitId()) {
            $query = array(
                'path' => 'organizationUnits',
                'query' => array(
                    'q' => "id='{$this->getOrganizationUnitId()}'"
                ),
            );
            if ($data = $this->_getApi()->queryOne($query)) {
                return print_r($data["leadForwardEmail"], true);
            }
        }
        return null;
    }

    public function getNewDevelopmentDeals() {
      $query = array(
        'path' => 'newDevelopmentDeals',
        'query' => array(
          'q' => "projectDealId='{$this->getId()}'"
        )
      );

      if ($data = $this->_getApi()->query($query)) {
        if (!empty($data)) {
          $newDevelopmentDeals = array_map(function($newDev) {
            return $newDev['dealId'];
          }, $data);

          if ($newDevDeals = ($this->_getApi()->query(array(
            'path' => 'deals',
            'query' => array(
              'q' => "id IN ('" . join("','", $newDevelopmentDeals) . "')"
            )
          )))) {
              $newDevDealsData = array();
              foreach ($newDevDeals as $key => $value) {
                array_push($newDevDealsData, new Mspecs_Model_Deal($value));
              }
              return new Mspecs_Collection($newDevDealsData);
          }
        }
      }
      return null;
    }

    public function getTeamRoles() {
        //fetch each teamRole through dealId
        $query = array(
            'path' => 'teamRoles',
            'query' => array(
                'q' => "dealId='{$this->getId()}'"
            )
        );
        //returns an array with all elements from prev query after applying contactId
        if ($teamRoles = $this->_getApi()->query($query)) {
            $contactIds = array();
            if (!empty($teamRoles)) {
                $contactIds = array_map(function($obj) {
                    return $obj['contactId'];
                }, $teamRoles);
                $query = "id IN ('" . join("','", $contactIds) . "')";
                if($contacts = ($this->_getApi()->query(array(
                    'path' => 'contacts',
                    'query' => array(
                        'q' => $query
                    )
                )))) {
                $brokers = array();
                foreach ($contacts as $key => $contact) {
                    foreach ($teamRoles as $key => $teamRole) {
                        if ($teamRole['contactId'] == $contact['id'] && $teamRole['isPublic'] == true) {
                            $contact['role'] = $teamRole['role'];
                            $contact['workTitle'] = $teamRole['workTitle'];
                            array_push($brokers, new Mspecs_Model_Contact($contact));
                        }
                    }
                }
                return new Mspecs_Collection($brokers);
                }
            }
        }

        return null;
    }

    //Gets all the people that are involved with the deal except the main broker
    public function getAssistant() {
        if($contacts = $this->getTeamRoles()) {
            foreach($contacts as $contact) {
                if($contact->getId() != $this->getMainBrokerContactId()) {
                    return $contact;
                }
            }
        }

        return null;
    }


    public function getGoogleStaticMapKey()
    {
        $config = MSPECS::getConfig()->get();
        $googleStaticMapKey = $config['googleStaticMapKey'];
        return $googleStaticMapKey;
    }

    public function getUseGoogleCaptcha()
    {
        $config = MSPECS::getConfig()->get();
        $useGoogleCaptcha = $config['useGoogleCaptcha'];
        return $useGoogleCaptcha;
    }

    public function getCaptchaPublic()
    {
        $config = MSPECS::getConfig()->get();
        $googleCaptcha = $config['googleCaptcha'];
        return $googleCaptcha['siteKey'];
    }

    public function getBidding()
    {
        if($this->getBiddingId()) {
            $query = array(
                'path' => 'biddings',
                'query' => array(
                    'q' => "id='{$this->getBiddingId()}'"
                ),
            );

            if ($data = $this->_getApi()->queryOne($query)) {
                return new Mspecs_Model_Bidding($data);
            }
        }
        return null;
    }

    //get list of the brokers for estate
    public function getBrokers()
    {
        if ($brokers = $this->_getApi()->getBrokers()) {
            return $brokers;
        }
        return null;
    }

    //get company logo
    public function getCompanyLogo()
    {
        return $this->_getApi()->getCompany()->getLogo();
    }

    //get main broker of the deal
    public function getMainBroker()
    {
        if($mainBrokerId = $this->getMainBrokerContactId()) {
            if($brokerContactId = $this->_getApi()->getContactById($mainBrokerId)) {
                return $brokerContactId;
            }
        }
        return null;
    }

    public function getFiles()
    {
        $files = array();
        if (!$this->_files) {
            $query = array(
                'path' => 'filesInDeals',
                'query' => array(
                    'q' => "dealId='{$this->getId()}'"
                ),
            );
            //files in deals object
            if ($data = $this->_getApi()->query($query)) {
                if(is_array($data) && !empty($data)) {
                    $fileIds = array_map(function($obj) { return $obj['fileId'];}, $data);
                    if(!empty($fileIds)) {
                        $queryFiles = array(
                            'path' => 'files',
                            'query' => array(
                                'q' => "id IN ('" . join("','", $fileIds) . "')",
                                'sort' => 'displayOrder'
                            )
                        );

                        if($filesData = $this->_getApi()->query($queryFiles)) {
                            if(is_array($filesData) && !empty($filesData)) {
                                foreach($filesData as $file) {
                                    $files[] = new Mspecs_Model_File($file);
                                }
                            }
                        }
                    }
                }
            }
        }
        return new Mspecs_Collection($files);
    }

    public function getFileIds() {
        $query = array(
            'path' => 'filesInDeals',
            'query' => array(
                'q' => "dealId='{$this->getId()}'"
            ),
        );

        //files in deals object
        if ($data = $this->_getApi()->query($query)) {
            return array_map(function ($obj) { return $obj['fileId']; }, $data);
        }
        return null;
    }

    public function getPublicFiles() {

        $fileIds = $this->getFileIds();
        $files = array();
        $displayOrder = array();
        if(is_array($fileIds)) {
            foreach($fileIds as $fileId) {
                if($file = $this->_getApi()->getFileById($fileId, true)) {
                    $files[]= $file;
                }
            }
            foreach ($files as $key => $row) {
                $displayOrder[$key] = $row->getDisplayOrder();
            }
            array_multisort($displayOrder, SORT_ASC, $files);
        }

        if(!empty($files))
            return $files;
        return null;
    }

    public function getTokenId() {
        if($tokenId = $this->getCommunicationTokenId()) {
            $query = array(
                'path' => 'communicationTokens',
                'query' => array(
                    'q' => "id='{$tokenId}'"));

            if($data = $this->_getApi()->queryOne($query)) {
                return $data['token'];
            }
        }

        return null;
    }

    public function getMainEstate()
    {
        if ($this->getMainEstateId()) {
            return $this->_getApi()->getEstateById($this->getMainEstateId());
        }
        return null;
    }

    public function getUrls()
    {
        if (!$this->_urls) {

            $query = array(
                'path' => 'dealUrls',
                'query' => array(
                    'q' => "dealId='{$this->getId()}' and isPublished=true",
                    'sort' => 'displayOrder'
                ),
            );

            //files in deals object
            if ($data = $this->_getApi()->getUrlsByDealId($query)) {
                if (sizeof($data) > 0) {
                    $this->_urls = $data;
                }
            }
        }

        if(!empty($this->_urls))
            return $this->_urls;
        return null;
    }

    public function getViewUrl()
    {
        return MSPECS::getHelper()->getUrl(array(
                    'type' => 'deal',
                    'deal_id' => $this->getId()
                ));
    }

    public function getImageViewUrl() {
        return MSPECS::getHelper()->getUrl(array(
            'type' => 'deal',
            'deal_id' => $this->getId(),
            'property-image' => true
        ));
    }

    public function getObjectType() {
        if($id = $this->getObjectTypeId()) {
            return $this->_getApi()->getObjectType($id);
        }
    }

    public function getOrganization() {
        if($organization = $this->_getApi()->getCompany()) {
            return $organization;
        }
        return null;
    }

    protected function getSlotStatus() {
        $api = $this->_getApi();
        $api->setNoCacheCallAndNoSchema(true);
        $this->_slotStatus = $api->getSlotStatus($this->getId());
        $this->_getApi()->setNoCacheCallAndNoSchema(false);
    }

    public function getVisitingDates() {
        if (empty($this->_viewings)) {
            //get today date if proper format
            $today = strtotime(date('M j, Y'));

            if ($viewings_ = $this->_getApi()->getViewings($this->getId())) {
                foreach ($viewings_ as $view) {
                    $timespan = strtotime($view->getStartDate());
                    if(($timespan>=$today && !$view->getContactBrokerForViewing())
                        || $view->getContactBrokerForViewing()
                        || $view->getDisplayOnlyViewingDescription()
                        || $view->getViewingType() == "ENUMS_VIEWINGS_TYPE_COMMENT_ONLY"
                    ) {
                        $this->_viewings[] = $view;
                    }
                }
            }
        }

        if ($this->_viewings) {
            foreach ($viewings_ as $view) {
                if ($view->getViewingType() == "ENUMS_VIEWINGS_TYPE_TIME_SLOTS") {
                    $this->getSlotStatus();
                    break;
                }
            }
        }

        return $this->_viewings;
    }

    public function getForeignCurrency(){
        return $this->_currencyForeign;
    }

    public function getCrncy(){
        return $this->_currency;
    }

    public function getWebsiteOption($option){
        return $this->_getApi()->getWebsiteOption($option);
    }
}
