<?php

class Mspecs_Model_Estate extends Mspecs_Model
{
    protected $sumCommercialOtherIncome = 0;
    protected $totalMortgage = 0;
    protected $countMortgage = 0;
    protected $_objectType = null;

    public function getLocation() {
        if (!$this->getLocationDescriptionId()) {
            return null;
        }

        $query = array(
            'path' => 'locationDescriptions',
            'query' => array(
                'q' => "id='{$this->getLocationDescriptionId()}'"
            )
        );

        if ($data = $this->_getApi()->queryOne($query)) {
            if ($data['fileId']) {
                $fileQuery = array(
                    'path' => 'files',
                    'query' => array(
                        'q' => "id='{$data['fileId']}'"
                    )
                );
                if ($fileData = $this->_getApi()->queryOne($fileQuery)) {
                    return new Mspecs_Model_Image($fileData);
                }
            }
        }

        return null;
    }

    public function getImages($category = null)
    {
        //not wanted default categories
        $categories = array(
//            "ENUMS_IMAGES_IN_ESTATES_CATEGORY_EXTERIOR" => true,
//            "ENUMS_IMAGES_IN_ESTATES_CATEGORY_INTERNAL" => true,
//            "ENUMS_IMAGES_IN_ESTATES_CATEGORY_MAP" => true
              "ENUMS_IMAGES_IN_ESTATES_CATEGORY_FLOOR_PLAN" => true
        );
        $q = "estateId='{$this->getId()}'";
        if ($category) {
            $q .= " and category='{$category}'";
        }

        $query = array(
            'path' => 'imagesInEstates',
            'query' => array(
                'q' => $q,
            ),
        );

        $files = array();

        if ($data = $this->_getApi()->query($query)) {
            //remove with unwanted categories for default.
            //Filters out all the images that should be displayed under the images section. For example the floor plan gets filtered out since it should not be displayed under the images section.
            //It gets display under it's own floor plan section instead.
            if(!$category) {
                $imagesInEstate = array();
                foreach($data as $imageInEstate) {
                    if(!isset($categories[$imageInEstate['category']])) {
                        $imagesInEstate[] = $imageInEstate;
                    }
                }
                $data = $imagesInEstate;
            }
            $fileIds = array_map(function($obj) { return $obj['fileId'];}, $data);
            if(!empty($fileIds)) {
                $queryFiles = array(
                    'path' => 'files',
                    'query' => array(
                        'q' => "id IN ('" . join("','", $fileIds) . "')",
                        'sort' => 'displayOrder'
                    )
                );
                if($filesData = $this->_getApi()->query($queryFiles)) {
                    if(is_array($filesData)) {
                        foreach($filesData as $file) {
                            $files[] = new Mspecs_Model_Image($file);
                        }
                    }
                }
            }
        }
        return new Mspecs_Collection($files);
    }

    public function hasLift() {
        $liftEnum = $this->getHasLiftSetting();

        if ($liftEnum == 'ENUM_HAS_LIFT_SETTING_YES') {
            return 'Ja';
        } else if ($liftEnum == 'ENUM_HAS_LIFT_SETTING_NO') {
            return 'Nej';
        } else {
            return null;
        }
    }

    public function getTransferFeePaidByValue() {
        if (!$this->getTransferFeePaidBy()) {
            return null;
        }

        switch (($this->getTransferFeePaidBy())) {
            case "ENUMS_HOUSING_ASSOCIATION_TRANSFER_FEE_PAID_BY_BUYER":
                return 'Köpare';
            case "ENUMS_HOUSING_ASSOCIATION_TRANSFER_FEE_PAID_BY_SELLER":
                return 'Säljare';
            case "ENUMS_HOUSING_ASSOCIATION_TRANSFER_FEE_PAID_BY_NO_FEE":
                return 'Ingen avgift';
            default:
                return null;
        }
    }

    public function getPublicImages($category = null)
    {
        //Gets all images that exist on an estate
        if ($allImages = $this->getImages($category)) {
            $images = array();
            //Filters out the images that are marked as published
            foreach ($allImages as $image) {
                if ($image->load()->getIsPublished()) {
                    $images[] = $image;
                }
            }
            ksort($images);
            if (sizeof($images) > 0) {
                return new Mspecs_Collection($images);
            }
        }
        return null;
    }

    public function getMortgages() {
        $query = array(
            'path' => 'mortgages',
            'query' => array(
                'q' => "estateId='{$this->getId()}'",
                'sort' => 'displayOrder'
            )
        );
        $mortgages = array();
        if($data = $this->_getApi()->query($query)) {
            foreach($data as $mortgage) {
                //calculate total mortgage
                $this->totalMortgage += (float) $mortgage['amount'];
                $this->countMortgage++;
                $mortgages[] = new Mspecs_Model($mortgage);
            }
        }
        return new Mspecs_Collection($mortgages);
    }

    public function getSumCommercialOtherIncome() {
        if($this->$sumCommercialOtherIncome == 0) {
            $this->getCommercialOtherIncomes();
        }
        return $this->$sumCommercialOtherIncome;
    }

    public function getTotalMortgage() {
        if($this->totalMortgage == 0) {
            $this->getMortgages();
        }
        return $this->totalMortgage;
    }

    public function getCountMortgage() {
        if($this->countMortgage==0) {
            $this->getMortgages();
        }
        return $this->countMortgage;
    }

	public function getRooms()
    {

        if($rooms = $this->_getApi()->getRoomsByEstate($this->getId())) {

            if(sizeof($rooms)>0) {
                return $rooms;
            }
        }
        return null;
    }

    public function getPremises()
    {

        if($premises = $this->_getApi()->getPremisesByEstate($this->getId())) {

            if(sizeof($premises)>0) {
                return $premises;
            }
        }
        return null;
    }

    public function getCountryName() {
        if($this->getCountryId()) {
            $query = array(
                'path' => 'countries',
                'query' => array(
                    'q' => "id='{$this->getCountryId()}'"));
            if($country = $this->_getApi()->queryOne($query)) {
                if(is_array($country)) {
                    return $country['name'];
                }
            }
        }
        return null;
    }

    public function getTenants() {
        $query = array('path' => 'estateTenants',
                'query' => array('q' => "estateId='{$this->getId()}'"));

        $tenantsList = array();
        if ($tenants = $this->_getApi()->query($query)) {
            foreach ($tenants as $key => $value) {
                array_push($tenantsList, new Mspecs_Model($value));
            }
        }
        if (empty($tenants)) {
            return [];
        } else {
            return new Mspecs_Collection($tenantsList);
        }
    }

    public function getEstateTenantTotal() {
        // get estate tenants
        $query = array('path' => 'estateTenants',
                'query' => array('q' => "estateId='{$this->getId()}'"));

        if ($tenants = $this->getTenants()) {
            if (empty($tenants)) {
                return null;
            }

            // get estate tenant object statuses
            $objectStatuses = $this->_getApi()->query(array('path' => 'estateTenantsObjectTypesStatus'));
            $objectTypes = $this->_getApi()->query(array('path' => 'estateTenantsObjectTypes'));

            $objectStatusesIds = $this->_setObjectById($objectStatuses);
            $objectTypeIds = $this->_setObjectById($objectTypes);

            $sumTenant = array('area' => 0, 'income' => 0);

            $sum = array(
                'rented' => $sumTenant,
                'vacant' => $sumTenant,
                'reserved' => $sumTenant,
                'numberOfTenants' => 0,
                'numberOfHousing' => 0,
                'numberOfPremises' => 0,
                'totalArea' => 0,
                'totalIncome' => 0
            );
            // prepare rented tenants
            foreach ($tenants as $key => $tenantModel) {
                $sum['numberOfTenants']++;
                $area = 0;
                if ($objectTypeIds[$tenantModel->getEstateTenantsObjectTypeId()]["name"] == 'ESTATE_TENANTS_OBJECT_TYPES_PREMISES') {
                    $sum['numberOfPremises']++;
                    if ($tenantModel->getArea()) {
                        $area = $tenantModel->getArea();
                    } else {
                        $area = 0;
                    }
                } else {
                    $sum['numberOfHousing']++;
                    if ($tenantModel->getLivingArea()) {
                        $area = $tenantModel->getLivingArea();
                    } else {
                        $area = 0;
                    }
                }

                // MSPECS::getHelper()->printJson($objectStatusesIds);
                // echo $tenantModel->getStatus() . '<br/>';
                if ($tenantModel->getStatus())
                switch ($objectStatusesIds[$tenantModel->getStatus()]['name']) {
                    case 'ESTATE_TENANTS_OBJECTY_TYPES_STATUS_RENTED':
                        $sum['rented']['income'] += $tenantModel->getIncome();
                        $sum['rented']['area'] += $area;
                        $sum['totalArea'] += $area;
                        $sum['totalIncome'] += $tenantModel->getIncome();
                        break;
                    case 'ESTATE_TENANTS_OBJECTY_TYPES_STATUS_VACANT':
                        $sum['vacant']['income'] += $tenantModel->getIncome();
                        $sum['vacant']['area'] += $area;
                        $sum['totalArea'] += $area;
                        $sum['totalIncome'] += $tenantModel->getIncome();
                        break;

                    case 'ESTATE_TENANTS_OBJECTY_TYPES_STATUS_RESERVED':
                        $sum['reserved']['income'] += $tenantModel->getIncome();
                        $sum['reserved']['area'] += $area;
                        $sum['totalArea'] += $area;
                        $sum['totalIncome'] += $tenantModel->getIncome();
                        break;
                }
            }

            return $sum;
        }
    }

    // converts object array into array with ids keys
    protected function _setObjectById($objectArray, $index = 'id') {
        $result = array();
        foreach ($objectArray as $key => $value) {
            $result[$value[$index]] = $value;
        }

        return $result;
    }

    public function getObjectTypeSubTypeText() {
        if ($this->getObjectSubTypeId()) {
          return $this->getSubObjectType();
        } else {
          return $this->getObjectType();
        }
    }

    public function getImageByOrder($order)
    {
        if ($images = $this->getImages()) {
            foreach ($images as $image) {
                if ($image->load()->getDisplayOrder() == $order) {
                    return $image;
                }
            }
        }
        return null;
    }

    public function inMonth($amount) {
        return round($amount / 12);
    }

    public function economyTenant($amount) {
        if($this->getObjectType()=="OBJECT_TYPE_TENANT_OWNERSHIP") {
            return $this->inMonth($amount);
        } else {
            return $amount;
        }
    }

    public function period() {
        global $translations;
        if($this->getObjectType()=="OBJECT_TYPE_TENANT_OWNERSHIP") {
            return "/" . mb_strtolower($translations["MONTH"], "UTF-8");
        } else {
            return "/" . mb_strtolower($translations["YEAR"], "UTF-8");
        }
    }

    public function getMainImage()
    {
        if ($imageId = $this->getMainImageFileId()) {
            return new Mspecs_Model_Image($imageId);
        }
        return null;
    }

    public function getBuildings() {
        if($this->getObjectTypeId()) {
            if($this->getObjectType()=='OBJECT_TYPE_COMMERCIAL' || $this->getObjectType()=='OBJECT_TYPE_FARMING') {
                if($buildings = $this->_getApi()->getBuildings($this->getId(), $this->getObjectType())) {
                    //Checks if there are any buildings
                    if(sizeof($buildings)>0) {
                        return $buildings;
                    }
                }
            }
        }
        return null;
    }

    public function getValuationUnits() {
        if($this->getObjectType()=='OBJECT_TYPE_COMMERCIAL') {
            if($buildings = $this->_getApi()->getValuationUnits($this->getId())) {
                if(sizeof($buildings)>0) {
                    return $buildings;
                }
            }
        }
        return null;
    }

    public function getFarmingSpecifics() {
        if($this->getObjectType()=='OBJECT_TYPE_FARMING') {
            if($farmingSpecifics = $this->_getApi()->getFarmingSpecifics($this->getId())) {
                if(sizeof($farmingSpecifics)>0) {
                    return $farmingSpecifics;
                }
            }
        }
        return null;
    }

    public function getAreas() {
        if($objectId = $this->getObjectTypeId()) {
            if($this->getObjectType()==='OBJECT_TYPE_FARMING') {
                if($areas = $this->_getApi()->getAreas($this->getId())) {
                    if(sizeof($areas)>0) {
                        return $areas;
                    }
                }
            }
        }
        return null;
    }

    public function getEnergyStatusText() {
        $energyStatus = null;
        switch($this->getEnergyStatus()) {
            case "ENUMS_EnergyStatus_TYPE_ENERGY_USE_IS_NOT_NEEDED":
                $energyStatus = "Energideklaration behövs ej";
                break;
            case "ENUMS_EnergyStatus_TYPE_ENERGY_USE_IS_MADE":
                $energyStatus = "Energideklaration är utförd";
                break;
            case "ENUMS_EnergyStatus_TYPE_ENERGY_USE_IN_PROGRESS":
                $energyStatus = "Energideklaration pågår";
                break;
            case "ENUMS_EnergyStatus_TYPE_ENERGY_AUDITS_NOT_PERFORMED":
                $energyStatus = "Energideklaration ej utförd";
                break;
            case "ENUMS_EnergyStatus_TYPE_ENERGY_AUDITS_ARE_COMMISSIONED":
                $energyStatus = "Energideklaration är beställd";
                break;
        }
        return $energyStatus;
    }

    public function getEnergyClassOnly() {
        if($energyClass = $this->getEnergyClass())
            return substr($energyClass, -1);
    }

    public function getObjectType() {
        if(!$this->_objectType) {
            if($id = $this->getObjectTypeId()) {
                $this->_objectType = $this->_getApi()->getObjectType($id);
            }
        }
        return $this->_objectType;
    }

    public function getSubObjectType() {
        $id = $this->getObjectSubTypeId();
        if (!$id)
            return "";

        if($data = $estateSubObject = $this->_getApi()->queryOne(array(
            'path' => 'objectSubTypes',
            'query' => array(
                'q' => "id='{$id}'"
            ),
        ))) {
            //Return the custom subtype if there is one. Otherwise the specified subtype.
            $subTypeText = $this->getObjectSubTypeText();
            return $subTypeText ? $subTypeText : $data["name"];
        }
        return "";
    }

    public function getBusinessTradeTypeValue() {
        if($value = $this->getBusinessTradeType()) {
            $tradeType = '';
            switch($value) {
                case "ENUM_ESTATE_BUSINESS_TRADE_TYPE_SHOPS";
                    $tradeType = 'Butiker';
                    break;
                case "ENUM_ESTATE_BUSINESS_TRADE_TYPE_CONSTRUCTION":
                    $tradeType = 'Byggverksamhet';
                    break;
                case "ENUM_ESTATE_BUSINESS_TRADE_TYPE_HOTEL/TOURIST":
                    $$tradeType = 'Hotell/Turistföretag';
                    break;
                case "ENUM_ESTATE_BUSINESS_TRADE_TYPE_RESTAURANTS/CAFES":
                    $tradeType = 'Restauranger/Caféer';
                    break;
                case "ENUM_ESTATE_BUSINESS_TRADE_TYPE_MANUFACTURING":
                    $tradeType = 'Tillverkande företag';
                    break;
                case "ENUM_ESTATE_BUSINESS_TRADE_TYPE_SERVICE_COMPANY":
                    $tradeType = 'Tjänsteföretag';
                    break;
                case "ENUM_ESTATE_BUSINESS_TRADE_TYPE_OTHER":
                    $tradeType = 'Övrigt';
                    break;
                case "ENUM_ESTATE_BUSINESS_TRADE_TYPE_COMMERCE/E-COMMERCE":
                    $tradeType = 'Handel/e-handel';
                    break;
                case "EENUM_ESTATE_BUSINESS_TRADE_TYPE_CARE_AND_HEALTH":
                    $tradeType = 'Vård & Hälsa';
                    break;
                case "ENUM_ESTATE_BUSINESS_TRADE_TYPE_CARRIER":
                    $tradeType = 'Transportföretag';
                    break;
            }
            return $tradeType;
        }
        return "";
    }

    public function getAssessValue() {
        if($this->getAssessValueTotal()) {
            return $this->getAssessValueTotal();
        } else {
            return intval($this->getAssessValueLand()) + intval($this->getAssessValueBuilding());
        }
    }

    public function getMainPurposes() {
        if($farmings = $this->_getApi()->getFarmingMainPurposes($this->getId())) {
            return $farmings;
        }
        return null;
    }

    public function getOperatingCosts() {
        $amounts = array();
        $query = array(
            'path' => 'estateOperatingCosts',
            'query' => array(
                'q' => "estateId='{$this->getId()}'"
            ),
        );
        if($operatingCosts = $this->_getApi()->query($query)) {
            if(!empty($operatingCosts)) {
                $amountIds = array_map(function($obj) { return $obj['amountId']; }, $operatingCosts);
                if(!empty($amountIds)) {
                    $queryAmounts = array(
                        'path' => 'amounts',
                        'query' => array(
                            'q' => "id IN ('" . join("','", $amountIds) . "')"
                        )
                    );
                    if($amountsData = $this->_getApi()->query($queryAmounts)) {
                        if(is_array($amountsData)) {
                            foreach($amountsData as $amount) {
                                $amounts[] = new Mspecs_Model_Amount($amount);
                            }
                        }
                    }
                }
            }
        }
        return new Mspecs_Collection($amounts);
    }

    public function getCommercialOtherIncomes() {
        $amounts = array();
        $query = array(
            'path' => 'commercialOtherIncomes',
            'query' => array(
                'q' => "estateId='{$this->getId()}'"
            ),
        );
        if($commercialOtherIncomes = $this->_getApi()->query($query)) {
            if(!empty($commercialOtherIncomes)) {
                $amountIds = array_map(function($obj) { return $obj['amountId']; }, $commercialOtherIncomes);
                if(!empty($amountIds)) {
                    $queryAmounts = array(
                        'path' => 'amounts',
                        'query' => array(
                            'q' => "id IN ('" . join("','", $amountIds) . "')"
                        )
                    );
                    if($amountsData = $this->_getApi()->query($queryAmounts)) {
                        if(is_array($amountsData)) {
                            foreach($amountsData as $amount) {
                                $this->$sumCommercialOtherIncome += (float) $amount['amount'];
                                $amounts[] = new Mspecs_Model_Amount($amount);
                            }
                        }
                    }
                }
            }
        }
        return new Mspecs_Collection($amounts);
    }

    public function getPatios() {
        if($patios = $this->_getApi()->getPatios($this->getId())) {
            return $patios;
        }
        return null;
    }

    public function getParkingSpaces() {
        if($parkingSpaces = $this->_getApi()->getParkingSpaces($this->getId())) {
            if(count($parkingSpaces)>0) {
                return $parkingSpaces;
            }
        }
        return null;
    }

    public function getViewUrl()
    {
        return MSPECS::getHelper()->getUrl(array(
                    'type' => 'estate',
                    'estate_id' => $this->getId(),
                ));
    }

    public function getHousingAssociation() {
        if($housingAsossiacionId = $this->getHousingAssociationId()) {
            return $this->_getApi()->getHousingAsociationById($housingAsossiacionId);
        }
        return null;
    }

    public function getPremiseExtras() {
        $amounts = array();
        $query = array(
            'path' => 'estatePremiseEconomyAmounts',
            'query' => array(
                'q' => "estateId='{$this->getId()}'"
            ),
        );
        if($data = $this->_getApi()->query($query)) {
            //retrieved array of relations table
            //get all amount ids on it
            if(is_array($data)) {
                $amountIds = array_map(function ($obj) { return $obj['amountId']; }, $data);
                if(!empty($amountIds)) {
                    $queryAmounts = array(
                        'path' => 'amounts',
                        'query' => array(
                            'q' => "id IN ('" . join("','", $amountIds) . "')"
                        )
                    );
                    if($amountsData = $this->_getApi()->query($queryAmounts)) {
                        if(is_array($amountsData)) {
                            foreach($amountsData as $amount) {
                                $amounts[] = new Mspecs_Model_Amount($amount);
                            }
                        }

                        return new Mspecs_Collection($amounts);
                    }
                }
            }
        }

        return null;
    }

    public function getPremiseIncludedCost() {
        $rentSum = 0;
        if($extras = $this->getPremiseExtras()) {
            foreach($extras as $extra) {
                $rentSum += $extra->getAmount();
            }
        }
        $rentSum += $this->getMonthlyRent() * 12;
        return $rentSum;
    }

    public function getEastemensAndDetails() {
      if ($this->getId()) {
        return $this->_getApi()->getEasementsAndDetailsByEstateId($this->getId());
      }
      return null;
    }

    public function showBuildingSection() {
        $showBuilding = false;
        $fieldsToCheck = array('generalInformation', 'landComment', 'buildYear', 'renovateYear', 'renovateDescription', 'ventilationType', 'ventilationInspectionDate',
        'windowsType', 'chimneyType', 'drainType', 'waterType', 'roofingType', 'radonType', 'foundationType', 'heatingType', 'heatingPlantCondition',
        'heatingPlantAge', 'heatingPlantBrand', 'heatingComment', 'electricalStemsCondition', 'mainFuse', 'electricSafety',
        'electricalComment', 'hasBurglarAlarm', 'internetSummary', 'hasCableTv', 'patioSummary', 'otherBuildings', 'faultsAndDefectsComment',
        'buildingComment', 'otherAreasComment', 'upcomingRenovationsDescription'
      );

      foreach ($fieldsToCheck as $key => $value) {
        if ($this->getData($value) != null && $this->getData($value) != '') {
          $showBuilding = true;
          break;
        }
      }
      return $showBuilding;
    }

    public function getAllIncludedInRent() {
        global $translations;
        $includedInRentTextArray = array();
        if ($this->getIsHeatIncluded()) {
            array_push($includedInRentTextArray, $translations['IS_HEAT_INCLUDED']);
        }

        if ($this->getIsWaterIncluded()) {
            array_push($includedInRentTextArray, $translations['IS_WATER_INCLUDED']);
        }

        if ($this->getIsHotWaterIncluded()) {
            array_push($includedInRentTextArray, $translations['IS_HOT_WATER_INCLUDED']);
        }

        if ($this->getIsColdWaterIncluded()) {
            array_push($includedInRentTextArray, $translations['IS_COLD_WATER_INCLUDED']);
        }

        if ($this->getIsElectricityIncluded()) {
            array_push($includedInRentTextArray, $translations['IS_ELECTRICITY_INCLUDED']);
        }

        if ($this->getIsGarageIncluded()) {
            array_push($includedInRentTextArray, $translations['IS_GARAGE_INCLUDED']);
        }

        if ($this->getIsCarportIncluded()) {
            array_push($includedInRentTextArray, $translations['IS_CARPORT_INCLUDED']);
        }

        if ($this->getIsCableTVIncluded()) {
            array_push($includedInRentTextArray, $translations['IS_CABLE_TV_INCLUDED']);
        }

        if ($this->getIsInternetIncluded()) {
            array_push($includedInRentTextArray, $translations['IS_INTERNET_INCLUDED']);
        }

        if ($this->getIsCoolingIncluded()) {
            array_push($includedInRentTextArray, $translations['IS_COOLING_INCLUDED']);
        }

        if ($this->getIsParkingSpaceIncluded()) {
            array_push($includedInRentTextArray, $translations['IS_PARKING_SPACE_INCLUDED']);
        }

        if ($this->getIsCondominiumSupplementIncluded()) {
            array_push($includedInRentTextArray, $translations['IS_CONDOMINIUM_SUPPLEMENT_INCLUDED']);
        }
        
        if (sizeof($includedInRentTextArray)>0) {
            return join(', ', $includedInRentTextArray);
        }

        return null;
    }

    public function getEconomicTypeCode(){
        if($id = $this->getEstateEconomicTypeCodeId()){
            if($query = $this->_getApi()->queryOne(array(
                "path" => "estateEconomicTypeCodes",
                "query" => array(
                    "q" => "id='{$id}'"
                )
            ))){
                return isset($query["code"]) ? $query["code"] : null;
            }
        }
        return null;
    }

    public function getPremiseRent(){
        if($this->getObjectType()=='OBJECT_TYPE_PREMISE'){
            $rent = 0;
            $unit = "";
            global $translations;

            if($this->getPremiseRentIsNegotiable()){
                $rent = $translations['FIELD_ESTATES_PREMISERENTISNEGOTIABLE'];
            }
            else {
                $monthlyRent = $this->getMonthlyRent();
                $displayPremiseRentAs = $this->getDisplayPremiseRentAs();

                switch($displayPremiseRentAs){
                    case "ENUM_ESTATE_DISPLAY_PREMISE_RENT_AS_CURRENCY/MONTH":
                        $rent = $monthlyRent;
                        $unit = "currencyPerMonth";
                        break;
                    case "ENUM_ESTATE_DISPLAY_PREMISE_RENT_AS_CURRENCY/YEAR/M2":
                        $rent = $monthlyRent * 12 / $this->getTotalArea();
                        $unit = "currencyPerYearPerM2";
                        break;
                    default:
                        $rent = $monthlyRent * 12;
                        $unit = "currencyPerYear";
                        break;
                }
            }
            return array("rent" => $rent, "unit" => $unit);
        }
        return null;
    }
}
