<?php

class Mspecs_View_Helper{
    public static function formatNumber($number, $decimals = 0){
        return number_format($number, $decimals, ',', ' ');
    }

    public static function formatNumberOneDecimal($number) {
        // Replaces decimals if they are 0
        return str_replace(",0", "",number_format($number, 1, ',', ' '));
    }

    public static function convertSqmToHectare($value = 0){
        return $value / 10000;
    }

    public static function getFloors($floorNr, $totalNrOfFloors){
        if(!($floorNr && $totalNrOfFloors))
            return "";

        global $translations;
        return $floorNr . " " . lcfirst($translations['OF']) . " " . $totalNrOfFloors;
    }

    public static function formatDate($unformatedDate) {
        if($unformatedDate){
            $date = strtotime($unformatedDate);
            //formats the date to desired output
            $formatedDate = strftime("%e %B %Y", $date);
            //using strtolower to have moth name lowecase because strftime dont have that option
            $lowerCaseDate = strtolower($formatedDate);
            return $lowerCaseDate;
        }
        return null;
    }

    public static function getFieldVaule($field, string $fieldName, $defaultValue = ""){
        if(!is_array($field))
            return $defaultValue;

        if(array_key_exists($fieldName, $field))
            return $field[$fieldName];

        return $defaultValue;
    }

    public static function getFromTo($from, $to, $unit = ""){
        if($from || $to){
            if(!empty($unit)){
                $from = "$from $unit";
                $to = "$to $unit";
            }

            return "$from - $to";
        }

        return "";
    }

    public static function getRenderViewPath(){
        return MSPECS::getViewPath("renderObjectDescriptionFields");
    }

    public static function createLongTextBlock(string $title, string $text){
        return
            "<ul>
                <h2 class='mspecs-wp-property-information-heading'>$title</h2>
                <li>
                    <pre>$text</pre>
                </li>
            </ul>"
        ;
    }

    public static function fieldsHasValues(array $fields){
        foreach ($fields as $field) {
            $value = self::getFieldVaule($field, "value", $field);
            $displayCondition = self::getFieldVaule($field, "displayCondition", true);

            if($value && $displayCondition){
                return true;
            }
        }

        return false;
    }

    public static function getPremiseRentField(Mspecs_Model_Estate $estate): array{
        $premiseRent = $estate->getPremiseRent();
        if($premiseRent == null){
            return array();
        }

        $title = "";
        if($estate->getSubObjectType() == "OBJECTSUBTYPES_PREMISE_TENANT_PREMISE_FOR_SALE"){
            $title = "FEE";
        }
        else{
            $title = "RENT";
        }

        return array(
            $title => array(
                "value" => $premiseRent['rent'],
                "unit"  => $premiseRent['unit'],
                "displayCondition"  => !$estate->getPremiseHideRentOnMarketingPlaces()
            )
        );
    }

    public static function getUnits(Mspecs_Model_Deal $deal, Mspecs_Model_Estate $estate){
        global $translations;

        $year = mb_strtolower($translations["YEAR"], "UTF-8");
        $cubicForest = "m³sk";

        $units = array(
            "squareMeters"          => $translations["KVM"],
            "rooms"                 => $translations["ROOMS"],
            "farmingArea"           => $translations["FARMING_AREA_UNIT"],
            "currency"              => $deal->getCrncy(),
            "currencyPerPeriod"     => $deal->getCrncy() . $estate->period(),
            "foreignCurrency"       => $deal->getForeignCurrency(),
            "currencyPerMonth"      => $deal->getCrncy() . "/" . lcfirst($translations["MONTH"]),
            "currencyPerYear"       => $deal->getCrncy() . "/$year",
            "currencyPerYearPerM2"  => $deal->getCrncy() . "/$year/{$translations['M2']}",
            "kwhPerSquareMeter"     => "kWh/{$translations['M2']}",
            "kwhPerYear"            => "kWh/$year",
            "cubicForest"           => $cubicForest,
            "cubicForestPerHectarePerYear"  => "$cubicForest/{$translations['FARMING_AREA_UNIT']}/$year"
        );

        return $units;
    }

    public static function getUtilsFilePath($file){
        return MSPECS::getViewPath("utils/$file");
    }
}
?>
